import ast
import contextlib
import copy
from functools import lru_cache
import json
from inventory_provider import nokia
import netifaces
import os
import pathlib
import re
import tempfile

from lxml import etree
import pytest

import inventory_provider
from inventory_provider.tasks import worker
from inventory_provider import config


TEST_DATA_DIR = pathlib.Path(__file__).parent / "data"


@lru_cache()
def read_json_test_data(filename: str):
    return json.loads(TEST_DATA_DIR.joinpath(filename).read_text())


def without_cache(data: dict):
    data = copy.copy(data)

    def _is_cache(s):
        if s.startswith('classifier-cache'):
            return True
        if s.startswith('joblog'):
            return True
        return False
    keys_to_delete = filter(_is_cache, data.keys())
    for k in list(keys_to_delete):
        del data[k]
    return data


DB_DATA = read_json_test_data("router-info.json")
DB_DATA_NOCACHE = without_cache(DB_DATA)


@lru_cache
def _load_nokia_doc(filename):
    path = pathlib.Path(__file__).parent / "data/nokia" / filename
    return nokia.remove_xml_namespaces(etree.parse(path))


@pytest.fixture
def load_nokia_netconf_config():
    def _load_netconf_config(hostname):
        return _load_nokia_doc(f"{hostname}-netconf.xml")
    return _load_netconf_config


@pytest.fixture
def load_nokia_state_doc():
    def _load_netconf_config(hostname):
        return _load_nokia_doc(f"{hostname}-state.xml")
    return _load_netconf_config


@pytest.fixture
def data_config_filename():

    with tempfile.NamedTemporaryFile() as f:
        config = {
            "ssh": {
                "username": "uSeR-NaMe",
                "private-key": "private-key-filename",
                "known-hosts": "known-hosts=filename"
            },
            "nokia-ssh": {
                "username": "uSeR-NaMe",
                "password": "dummy-password"
            },
            "redis": {
                "hostname": "xxxxxx",
                "port": 6379,
                "socket_timeout": 2.8
            },
            "redis-databases": [0, 7],
            "ims": {
                "api": "ims_api",
                "username": "ims_username",
                "password": "ims_password"
            },
            "unmanaged-interfaces": [
                {
                    "address": "99.99.99.99",
                    "network": "99.99.99.0/24",
                    "interface": "ABC/0/0/0",
                    "router": "bogus.host.name"
                },
                {
                    "address": "999.999.999.99",
                    "network": "999.999.999.0/24",
                    "interface": "ZZZ/9/a/x:0.123",
                    "router": "another.bogus.host.name"
                }
            ],
            'gws-direct': {},
            'nren-asn-map': [
                {
                    "nren": "FOO",
                    "asn": 1930
                },
                {
                    "nren": "BAR",
                    "asn": 680
                },
                {
                    "nren": "BAT",
                    "asn": 2200
                },
                {
                    "nren": "BAZ",
                    "asn": 1853
                }
            ],
            "aai": {
                "discovery_endpoint_url": "https://smaple.discovery.endpoint",
                "inventory_provider": {
                    "client_id": "sample-client-id",
                    "secret": "sample-secret"
                }
            },
            "orchestrator": {
                "url": "https://orchestrator.url"
            }
        }

        config['gws-direct'] = read_json_test_data('gws-direct.json')

        f.write(json.dumps(config).encode('utf-8'))
        f.flush()
        yield f.name


@pytest.fixture
def data_config(data_config_filename):
    with open(data_config_filename) as f:
        return config.load(f)


class DictOverlay:
    """Since we're dealing with a very large dictionary from 'router-info.json' we only
    want to read that dictionary once and prevent mutating it during tests. We instead
    use this class to capture all mutations in additional layer that is reset every test

    Not thread safe
    """
    TOMBSTONE = object()  # record deletions

    def __init__(self, base_dict: dict) -> None:
        self.dict = base_dict
        self.overlay = {}

    def __getitem__(self, key):
        if key not in self.overlay:
            return self.dict[key]
        value = self.overlay[key]
        if value is self.TOMBSTONE:
            raise KeyError(key)
        return value

    def __setitem__(self, key, value):
        self.overlay[key] = value

    def __delitem__(self, key):
        self.overlay[key] = self.TOMBSTONE

    def __contains__(self, key):
        if key in self.overlay:
            return self.overlay[key] is not self.TOMBSTONE
        return key in self.dict

    def get(self, key, default=None):
        try:
            return self[key]
        except KeyError:
            return default

    def keys(self):
        deleted_keys = {k for k, v in self.overlay.items() if v is self.TOMBSTONE}
        return (self.dict.keys() | self.overlay.keys()) - deleted_keys

    def items(self):
        return ((key, self[key]) for key in self.keys())

    _missing = object

    def pop(self, key, default=_missing):
        try:
            value = self[key]
        except KeyError:
            if default is self._missing:
                raise
            return default

        self.overlay[key] = self.TOMBSTONE
        return value


class MockedRedis:
    def __init__(self, *args, **kwargs):
        self.db = DictOverlay(DB_DATA_NOCACHE)
        self.db['db:latch'] = json.dumps({
                'current': 0,
                'next': 0,
                'this': 0,
                'pending': False,
                'failure': False
            })

    def set(self, name, value):
        self.db[name] = value

    def get(self, name):
        value = self.db.get(name, None)
        if value is None:
            return None
        return value.encode('utf-8')

    def exists(self, name):
        return name in self.db

    def delete(self, key):
        if isinstance(key, bytes):
            key = key.decode('utf-8')

        if key in self.db:
            del self.db[key]

    def scan_iter(self, glob=None, count='unused'):
        if not glob:
            for k in list(self.db.keys()):
                yield k.encode('utf-8')

        m = re.match(r'^([^*]+)\*$', glob)
        assert m  # all expected globs are like this
        for k in list(self.db.keys()):
            if k.startswith(m.group(1)):
                yield k.encode('utf-8')

    def keys(self, glob=None):
        return list(self.scan_iter(glob))

    def flushdb(self):
        # only called from testing routes (hopefully)
        pass

    def execute(self):
        pass

    def pipeline(self, *args, **kwargs):
        return self


@pytest.fixture(scope="session")
def cached_test_data():
    return DB_DATA


@pytest.fixture
def flask_config_filename():
    with tempfile.NamedTemporaryFile() as f:
        f.write('ENABLE_TESTING_ROUTES = True\n'.encode('utf-8'))
        f.flush()
        yield f.name


@pytest.fixture
def mocked_redis(mocker):
    instance = MockedRedis()
    mocker.patch(
        'inventory_provider.tasks.common.redis.StrictRedis',
        return_value=instance)
    return instance


@pytest.fixture
def client(flask_config_filename, data_config_filename, mocked_redis):
    os.environ['FLASK_SETTINGS_FILENAME'] = flask_config_filename
    os.environ['INVENTORY_PROVIDER_CONFIG_FILENAME'] = data_config_filename
    app = inventory_provider.create_app(setup_logging=False)
    app.testing = True  # Show exceptions instead of generid 500 status
    with app.test_client() as c:
        yield c


NETIFACES_TEST_DATA_STRING = """{
    'lo0':  {{AF_INET}: [{'addr': '127.0.0.1', 'netmask': '255.0.0.0', 'peer': '127.0.0.1'}],
             {AF_INET6}: [{'addr': '::1', 'netmask': 'ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff/128', 'peer': '::1', 'flags': 0},
                 {'addr': 'fe80::1%lo0', 'netmask': 'ffff:ffff:ffff:ffff::/64', 'flags': 0}]},
    'eth0': {{AF_LINK}: [{'addr': '78:4f:43:76:73:ba'}],
             {AF_INET}: [{'addr': '83.97.92.239', 'netmask': '255.255.252.0', 'broadcast': '83.97.95.255'}],
             {AF_INET6}: [{'addr': 'fe80::250:56ff:fea1:8340', 'netmask': 'ffff:ffff:ffff:ffff::/64', 'flags': 1024},
                 {'addr': '2001:798:3::104', 'netmask': 'ffff:ffff:ffff:ffff::/64', 'flags': 1088}]}
}"""  # noqa E501


@pytest.fixture
def mocked_netifaces(mocker):
    s = NETIFACES_TEST_DATA_STRING
    for k, v in {
        'AF_INET': netifaces.AF_INET,
        'AF_INET6': netifaces.AF_INET6,
        'AF_LINK': netifaces.AF_LINK
    }.items():
        s = s.replace('{%s}' % k, str(v))
    data = ast.literal_eval(s)
    mocker.patch('netifaces.interfaces', lambda: data.keys())
    mocker.patch('netifaces.ifaddresses', lambda n: data[n])


@contextlib.contextmanager
def _mocked_db_connection(ignored):
    yield None


@pytest.fixture
def mocked_worker_module(
        mocker, mocked_redis, data_config_filename,
        cached_test_data, mocked_netifaces):

    os.environ['INVENTORY_PROVIDER_CONFIG_FILENAME'] = data_config_filename

    with open(data_config_filename) as f:
        worker.InventoryTask.config = config.load(f)

    def _mocked_send_event(*kargs, **kwargs):
        pass
    mocker.patch(
        'inventory_provider.tasks.worker.InventoryTask.send_event',
        _mocked_send_event)

    def _mocked_snmp_interfaces(hostname, community, logical_systems):
        return json.loads(cached_test_data['snmp-interfaces:' + hostname])
    mocker.patch(
        'inventory_provider.snmp.get_router_snmp_indexes',
        _mocked_snmp_interfaces)

    def _mocked_snmp_peerings_juniper(hostname, community, logical_systems):
        def _wanted(s):
            return re.match(r'^snmp-peerings:hosts:juniper\d.*', s)

        keys = filter(_wanted, cached_test_data.keys())
        return [json.loads(cached_test_data[k]) for k in keys]

    def _mocked_snmp_peerings_nokia(hostname, walk_community, poll_community):
        def _wanted(s):
            return re.match(r'^snmp-peerings:hosts:nokia\d.*', s)

        keys = filter(_wanted, cached_test_data.keys())
        return [json.loads(cached_test_data[k]) for k in keys]
    mocker.patch(
        'inventory_provider.snmp.get_peer_state_info_juniper',
        _mocked_snmp_peerings_juniper)
    mocker.patch(
        'inventory_provider.snmp.get_peer_state_info_nokia',
        _mocked_snmp_peerings_nokia)

    def _mocked_load_juniper_netconf_config(hostname, _):
        return etree.XML(cached_test_data['netconf:' + hostname])
    mocker.patch(
        'inventory_provider.juniper.load_config',
        _mocked_load_juniper_netconf_config)
